unit sSplash;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  T_Splash = class(TForm)
    ShowTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure ShowTimerTimer(Sender: TObject);
  private
    procedure CreateRegion(vVolume: Integer);
  public
    procedure SplashVolume(vVolume: Integer; vInterval: Integer);
    procedure CreateParams(var Params: TCreateParams); override;
  end;

const BrdOffs = 4;
  Offs = 7;
  TextTopOffs = 2;

implementation

procedure FillGradientRect(Canvas: TCanvas; Recty: TRect; fbcolor, fecolor: TColor; fcolors: Integer);
var
  i, j, h, w, fcolor: Integer;
  R, G, B: Longword;
  beginRGBvalue, RGBdifference: array[0..2] of Longword;
begin
  beginRGBvalue[0] := GetRvalue(colortoRGB(FBcolor));
  beginRGBvalue[1] := GetGvalue(colortoRGB(FBcolor));
  beginRGBvalue[2] := GetBvalue(colortoRGB(FBcolor));

  RGBdifference[0] := GetRvalue(colortoRGB(FEcolor)) - beginRGBvalue[0];
  RGBdifference[1] := GetGvalue(colortoRGB(FEcolor)) - beginRGBvalue[1];
  RGBdifference[2] := GetBvalue(colortoRGB(FEcolor)) - beginRGBvalue[2];

  Canvas.pen.Style := pssolid;
  Canvas.pen.mode := pmcopy;
  j := 0;
  h := recty.Bottom - recty.Top;
  w := recty.Right - recty.Left;

  for i := fcolors downto 0 do
  begin
    recty.Left := muldiv(i - 1, w, fcolors);
    recty.Right := muldiv(i, w, fcolors);
    if fcolors > 1 then
    begin
      R := beginRGBvalue[0] + muldiv(j, RGBDifference[0], fcolors);
      G := beginRGBvalue[1] + muldiv(j, RGBDifference[1], fcolors);
      B := beginRGBvalue[2] + muldiv(j, RGBDifference[2], fcolors);
    end;
    Canvas.Brush.Color := RGB(R, G, B);
    patBlt(Canvas.Handle, recty.Left, recty.Top, Recty.Right - recty.Left, h, patcopy);
    Inc(j);
  end;
end;

{$R *.DFM}

procedure T_Splash.FormCreate(Sender: TObject);
begin
  Top := 60;
  Left := 0;
  Width := Screen.Width;
  Height := 50;
end;

procedure T_Splash.SplashVolume(vVolume: Integer; vInterval: Integer);
begin
  ShowTimer.Enabled := False;
  CreateRegion(vVolume);
  ShowTimer.Interval := vInterval;
  Visible := True;
  ShowTimer.Enabled := True;
end;

procedure T_Splash.ShowTimerTimer(Sender: TObject);
begin
  ShowTimer.Enabled := False;
  Visible := False;
end;

procedure T_Splash.CreateRegion(vVolume: Integer);
var bm: TBitmap;
  HRgn: THandle;
  vStr: string;
begin
  bm := TBitmap.Create;
  bm.Height := Height;
  bm.Width := Screen.Width;

  BeginPath(bm.Canvas.Handle);


  bm.Canvas.Rectangle(0, 0, Screen.Width, Height);
  bm.Canvas.Brush.Color := clWhite;
  bm.Canvas.Rectangle(BrdOffs, BrdOffs, Screen.Width - BrdOffs, Height - BrdOffs);
  bm.Canvas.Brush.Color := clBlack;
  bm.Canvas.Rectangle(Offs, Offs, Offs + Round((Screen.Width - Offs * 2) * vVolume / 100), Height - Offs);

  vStr := IntToStr(vVolume) + ' %';
  bm.Canvas.Font.Name := 'Tahoma';
  bm.Canvas.Font.Size := 26;
  bm.Canvas.Font.Style := [fsBold];
  bm.Canvas.Brush.Style := bsClear;
  bm.Canvas.TextOut(Offs * 2, TextTopOffs, vStr);

  EndPath(bm.Canvas.Handle);

  HRgn := PathToRegion(bm.Canvas.Handle);
  SetWindowRgn(Handle, HRgn, true);

  FillGradientRect(Canvas, rect(Offs, Offs, Width, Height - Offs), clRed, clLime, 200);

  DeleteObject(HRgn);
  bm.Free;
end;

procedure T_Splash.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle := Params.ExStyle or WS_EX_TOOLWINDOW;
end;



end.

