unit sMAPI;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, sMAPIFunc, RXShell, Menus, ActnList, Placemnt, sRemoteClasses,
  ExtCtrls, ImgList;

type
  tcallbackprc = procedure(vCode: Integer);
  pcallbackprc = ^tcallbackprc;
  tsendmsgprc = procedure(hWnd: HWND; Msg: Cardinal; wParam, lParam: Integer);


  T_RSignalMAPI = class(TForm)
    TrayIcon: TRxTrayIcon;
    PopupMenu: TPopupMenu;
    ActionList1: TActionList;
    AcSetup: TAction;
    AcSetup1: TMenuItem;
    AcExit: TAction;
    N1: TMenuItem;
    N2: TMenuItem;
    AcPultKey: TAction;
    N3: TMenuItem;
    AntiRepeatTimer: TTimer;
    ImageList1: TImageList;
    AcAction: TAction;
    N4: TMenuItem;
    AcSite: TAction;
    procedure FormCreate(Sender: TObject);
    procedure AcExitExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AcPultKeyExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure AntiRepeatTimerTimer(Sender: TObject);
    procedure AcSetupExecute(Sender: TObject);
    procedure AcActionExecute(Sender: TObject);
  private
    FAllPultKeys: TAllPultKeys;
    FGlobalSettings: TGlobalSettings;
    FRemoteAppList: TRemoteAppList;
    procedure WMUser(var Msg: TMessage); message WM_USER;
    procedure ShowOSDText(vMsg1, vMsg2: string);
    procedure ShowOSDVolume(vVolume: Integer);
  public
    procedure ProcessKey(vCode: Integer);
  end;

const
  RTELSIGNAL_REMOTECODE = 1;
  RTELSIGNAL_SHOWSPLASHINFO_SWITCH = 2;
  RTELSIGNAL_HELP = 3;

  RTELSIGNAL_VOLUME_UP = 10;
  RTELSIGNAL_VOLUME_DOWN = 11;
  RTELSIGNAL_VOLUME_MUTE_SWITCH = 12;

  RTELSIGNAL_SELECTDLG_SHOW = 20;
  RTELSIGNAL_SELECTDLG_NEXT = 21;
  RTELSIGNAL_SELECTDLG_PREV = 22;
  RTELSIGNAL_SELECTDLG_CLOSE = 23;

  RTELSIGNAL_FILEOPEN_SHOW = 30;
  RTELSIGNAL_FILEOPEN_NEXT = 31;
  RTELSIGNAL_FILEOPEN_PREV = 32;
  RTELSIGNAL_FILEOPEN_EXEC = 34;

  RTELSIGNAL_SHUTDOWN = 50;

  RTELSIGNAL_CLOSEREMOTEAPP = 60;

  RTELSIGNAL_BSPLAY_SETPOS = 71;
  RTELSIGNAL_BSPLAY_PLAYPAUSE = 72;
  RTELSIGNAL_WINAMP_PLAYPAUSE = 73;

  RTELSIGNAL_SETMOUSE = 80;
  RTELSIGNAL_MOVEMOUSE = 81;
  RTELSIGNAL_MOUSECLICK = 82;
  RTELSIGNAL_KEYDOWN = 83;

var
  _RSignalMAPI: T_RSignalMAPI;

procedure SetCallbackProc(cbp: pcallbackprc); external 'Remote.dll';
procedure UseCallback(vCode: Integer);
procedure SplashInfo(vStr: PChar; vAddStr: PChar; vInterval: Integer); external 'OSDText.dll';
procedure SplashVolume(vVolume: Integer; vInterval: Integer); external 'OSDVolume.dll';
function InterceptSignal(vCode: Integer; vAppIndex: Integer): Boolean; external 'UserProc.dll';

implementation

{$R *.DFM}

procedure UseCallback(vCode: Integer);
begin
  _RSignalMAPI.ProcessKey(vCode);
end;

{ T_RSignalMAPI }


procedure T_RSignalMAPI.ProcessKey(vCode: Integer);
var FKeyMessage: string;
begin
  if not AntiRepeatTimer.Enabled then
  begin
    TrayIcon.Icon.Handle := LoadIcon(hInstance, 'PULT_ACT');
    if not FAllPultKeys.InterceptKey(vCode) then
    begin
      if not InterceptSignal(vCode, FRemoteAppList.CurrentAppIndex) then
      begin
        FKeyMessage := FRemoteAppList.ProcessMsg(vCode);
        ShowOSDText(FKeyMessage, '');
      end;
    end;
    AntiRepeatTimer.Enabled := True;
    TrayIcon.Icon.Handle := LoadIcon(hInstance, 'PULT');
  end;
end;

procedure T_RSignalMAPI.WMUser(var Msg: TMessage);
var FInfoMessage: string;
begin
  case Msg.WParam of
    RTELSIGNAL_REMOTECODE: ProcessKey(Msg.LParam);
    RTELSIGNAL_VOLUME_UP:
      begin
        ShowOSDVolume(SetVolume(Msg.LParam, FGlobalSettings.VolumeInc));
        Exit;
      end;
    RTELSIGNAL_VOLUME_DOWN:
      begin
        ShowOSDVolume(SetVolume(Msg.LParam, -FGlobalSettings.VolumeInc));
        Exit;
      end;
    RTELSIGNAL_VOLUME_MUTE_SWITCH: FInfoMessage := Mute;
    RTELSIGNAL_SELECTDLG_SHOW: FRemoteAppList.SelectRunApp(scSelect);
    RTELSIGNAL_SELECTDLG_NEXT: FRemoteAppList.SelectRunApp(scNext);
    RTELSIGNAL_SELECTDLG_PREV: FRemoteAppList.SelectRunApp(scPrev);
    RTELSIGNAL_SELECTDLG_CLOSE: FRemoteAppList.SelectRunApp(scCancel);
    RTELSIGNAL_FILEOPEN_SHOW: FRemoteAppList.SelectRunApp(scFileSelect);
    RTELSIGNAL_FILEOPEN_NEXT: FRemoteAppList.SelectRunApp(scFileNext);
    RTELSIGNAL_FILEOPEN_PREV: FRemoteAppList.SelectRunApp(scFilePrev);
    RTELSIGNAL_FILEOPEN_EXEC: FRemoteAppList.SelectRunApp(scFileExec);
    RTELSIGNAL_CLOSEREMOTEAPP: FRemoteAppList.CloseCurrentApp;
    RTELSIGNAL_SHUTDOWN: FInfoMessage := ShutDown;
    RTELSIGNAL_BSPLAY_SETPOS: FInfoMessage := BSPlayer_ChangePos(Msg.LParam);
    RTELSIGNAL_BSPLAY_PLAYPAUSE: FInfoMessage := BSPlayer_PlayPause;
    RTELSIGNAL_WINAMP_PLAYPAUSE: FInfoMessage := WinAMP_PlayPause;
    RTELSIGNAL_SHOWSPLASHINFO_SWITCH: FGlobalSettings.ShowSplashMsg := 1 - FGlobalSettings.ShowSplashMsg;
    RTELSIGNAL_HELP: FRemoteAppList.HelpDlg;
    RTELSIGNAL_SETMOUSE: FInfoMessage := MouseSetPos(Msg.LParam);
    RTELSIGNAL_MOVEMOUSE: FInfoMessage := MouseMoveDelta(Msg.LParam);
    RTELSIGNAL_MOUSECLICK: FInfoMessage := MouseClickAct(TMouseClickAct(Msg.LParam));
    RTELSIGNAL_KEYDOWN: FInfoMessage := KeyPressAct(Msg.LParam);
  else Exit;
  end;
  ShowOSDText('', FInfoMessage);
end;

procedure T_RSignalMAPI.ShowOSDText(vMsg1, vMsg2: string);
begin
  if (FGlobalSettings.ShowSplashMsg = 1) and (not FRemoteAppList.SelectDlgShowing)
    then SplashInfo(PChar(vMsg1), PChar(vMsg2), FGlobalSettings.SplashTime);
end;

procedure T_RSignalMAPI.ShowOSDVolume(vVolume: Integer);
begin
  if (FGlobalSettings.ShowSplashMsg = 1) then SplashVolume(vVolume, FGlobalSettings.SplashTime);
end;

procedure T_RSignalMAPI.FormCreate(Sender: TObject);
var IniFN: string;
begin
  SetCallbackProc(@UseCallback);
  IniFN := ChangeFileExt(Application.ExeName, '.ini');
  FAllPultKeys := TAllPultKeys.Create(Self, IniFN);
  FGlobalSettings := TGlobalSettings.Create(Self, IniFN);
  FRemoteAppList := TRemoteAppList.Create(Self, IniFN, FAllPultKeys);
  AntiRepeatTimer.Interval := FGlobalSettings.Interval;
  AntiRepeatTimer.Enabled := False;
  TrayIcon.Icon.Handle := LoadIcon(hInstance, 'PULT');
end;

procedure T_RSignalMAPI.AcExitExecute(Sender: TObject);
begin
  Close;
end;

procedure T_RSignalMAPI.FormDestroy(Sender: TObject);
begin
  FAllPultKeys.Free;
  FGlobalSettings.Free;
  FRemoteAppList.Free;
end;

procedure T_RSignalMAPI.AcPultKeyExecute(Sender: TObject);
begin
  FAllPultKeys.ShowEditDlg;
end;

procedure T_RSignalMAPI.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  FAllPultKeys.SaveSettings;
  FGlobalSettings.SaveSettings;
  FRemoteAppList.SaveSettings;
end;

procedure T_RSignalMAPI.AntiRepeatTimerTimer(Sender: TObject);
begin
  AntiRepeatTimer.Enabled := False;
end;

procedure T_RSignalMAPI.AcSetupExecute(Sender: TObject);
begin
  if FGlobalSettings.ShowEditDlg then
  begin
    AntiRepeatTimer.Interval := FGlobalSettings.Interval;
  end;
end;

procedure T_RSignalMAPI.AcActionExecute(Sender: TObject);
begin
  FRemoteAppList.ShowEditDlg;
end;


end.

