unit sRemoteClasses;

interface

uses contnrs, Placemnt, classes, Sysutils, sPultKeyDlg, controls, dxTl,
  Windows, Messages, sOptionsDlg, ShellAPI, Forms, stdctrls, sRemAppListDlg,
  Dialogs, sSelectRunApp, graphics, comctrls, sProcesses, RXShell, sHelp;

const MaxKeyIndex = 256;

type
  TSelectCtrl = (scNext, scPrev, scCancel, scSelect,
    scFileNext, scFilePrev, scFileSelect, scFileExec);

  //         ini
  TIniSaveClass = class(TComponent)
    constructor Create(AOwner: TComponent; aIniFileName: string; aIniSection: string);
    destructor Destroy; override;
  protected
    FIni: TFormStorage;
  public
    procedure SaveSettings; virtual;
    procedure ReadSettings; virtual;
    procedure SaveToGrid(aGrid: TdxTreeList); virtual;
    procedure ReadFromGrid(aGrid: TdxTreeList); virtual;
    function ShowEditDlg: Boolean; virtual;
  end;

  //  
  TGlobalSettings = class(TIniSaveClass)
    constructor Create(AOwner: TComponent; aIniFileName: string);
  public
    VolumeInc: Integer;
    Interval: Integer;
    SplashTime: Integer;
    ShowSplashMsg: Integer;
    procedure SaveSettings; override;
    procedure ReadSettings; override;
    function ShowEditDlg: Boolean; override;
  end;

  //  
  TPultKey = class
    Code: Integer;
    Descr: string;
  end;

  //   
  TAllPultKeys = class(TIniSaveClass)
    constructor Create(AOwner: TComponent; aIniFileName: string);
    destructor Destroy; override;
  private
    FPultKeysList: TObjectList;
    FGrid: TdxTreeList;
    FEditDlgShowing: Boolean;
  public
    function InterceptKey(vCode: Integer): Boolean;
    procedure SaveSettings; override;
    procedure ReadSettings; override;
    procedure SaveToGrid(aGrid: TdxTreeList); override;
    procedure ReadFromGrid(aGrid: TdxTreeList); override;
    function ShowEditDlg: Boolean; override;
    procedure AddPultKey(aCode: Integer; aDescr: string);
    procedure Clear;
  end;

  // 
  TRemoteApp = class(TIniSaveClass)
    constructor Create(AOwner: TComponent; aIniFileName: string; aAppName: string);
    destructor Destroy; override;
  private
    FKeysPoint: array[0..MaxKeyIndex] of Pointer;
    FSettingList: TObjectList;
    function GetPultKeyByCode(vCode: Integer): TPultKey;
    function GetAppByName(vAppName: string): TRemoteApp;
  public
    AppName: string;
    FilePathName: string;
    ParamExec: string;
    ClassName: string;
    WindowName: string;
    procedure SaveSettings; override;
    procedure ReadSettings; override;
    function ProcessMsg(vCode: Integer): string;
    procedure SaveToGrid(aGrid: TdxTreeList); override;
    procedure ReadFromGrid(aGrid: TdxTreeList); override;
  end;

  //   
  TRemoteSetting = class
    PultKey: Integer;
    PPultKey: TPultKey;
    Msg: Cardinal;
    LParam: Longint;
    WParam: Longint;
    Comment: string;
    Rept: Integer;
    MsgApp: string;
    PMsgApp: TRemoteApp;
  end;

  //  
  TRemoteAppList = class(TIniSaveClass)
    constructor Create(AOwner: TComponent; aIniFileName: string; vPultKeys: TAllPultKeys);
    destructor Destroy; override;
  private
    FPultKeys: TAllPultKeys;
    FAppList: TObjectList;
    FRemAppDlg: T_RemAppListDlg;
    FSelectRunApp: T_SelectRunApp;
    FLastOpenDir: string;
    FTrayIcon: TRxTrayIcon;
    FHelp: T_Help;
    procedure LoadAppsToCombo;
    procedure LoadAppInfoToForm;
    procedure LoadAppInfoFromForm;
    procedure AcSave(Sender: TObject);
    procedure FillAppItems;
    procedure ChangeToApp(vIndex: Integer);
    procedure CloseSelectRunAppDlg;
    procedure AcDel(Sender: TObject);
    procedure AcNew(Sender: TObject);
    procedure SaveLastDir;
  public
    CurrentApp: TRemoteApp;
    CurrentAppIndex: Integer;
    SelectDlgShowing: Boolean;
    procedure SaveSettings; override;
    procedure ReadSettings; override;
    function ProcessMsg(vCode: Integer): string;
    procedure AddAppToList(vApp: TRemoteApp); overload;
    procedure AddAppToList(vAppName, vFilePathName, vParamExec,
      vClassName, vWindowName: string); overload;
    function ShowEditDlg: Boolean; override;
    procedure ComboChange(Sender: TObject);
    procedure SelectRunApp(vSelCtrl: TSelectCtrl);
    procedure CloseCurrentApp;
    procedure HelpDlg;
  end;

const SelfAppName = '$RemoteControl$';

procedure AppTestRun(aApp: TRemoteApp);
function AppFindWindow(aApp: TRemoteApp): HWND;

implementation

function AppFindWindow(aApp: TRemoteApp): HWND;
begin
  Result := 0;
  if aApp.WindowName = '' then
  begin
    Result := FindWindow(PChar(aApp.ClassName), nil);
    Exit;
  end;
  if aApp.ClassName = '' then
  begin
    Result := FindWindow(nil, PChar(aApp.WindowName));
    Exit;
  end;
  Result := FindWindow(PChar(aApp.ClassName), PChar(aApp.WindowName));
end;

procedure AppTestRun(aApp: TRemoteApp);
begin
  if AppFindWindow(aApp) = 0 then
  begin
    if AnsiUpperCase(ExtractFileExt(aApp.FilePathName)) <> '.ICO' then
      ShellExecute(0, 'open',
        PChar(ExtractFileName(aApp.FilePathName)),
        PChar(aApp.ParamExec),
        PChar(ExtractFilePath(aApp.FilePathName)),
        SW_SHOWNORMAL);
  end;
end;

{ TAllPultKeys }

procedure TAllPultKeys.AddPultKey(aCode: Integer; aDescr: string);
var i: Integer;
  aPultKey: TPultKey;
begin
  for i := 0 to FPultKeysList.Count - 1 do
    if aCode = TPultKey(FPultKeysList.Items[i]).Code then
    begin
      Exit;
    end;
  aPultKey := TPultKey.Create;
  aPultKey.Code := aCode;
  aPultKey.Descr := aDescr;
  FPultKeysList.Add(aPultKey);
end;

procedure TAllPultKeys.Clear;
begin
  FPultKeysList.Clear;
end;

constructor TAllPultKeys.Create(AOwner: TComponent; aIniFileName: string);
begin
  FPultKeysList := TObjectList.Create(true);
  inherited Create(AOwner, aIniFileName, 'PultKeys');
end;

destructor TAllPultKeys.Destroy;
begin
  Clear;
  FPultKeysList.Free;
  inherited;
end;

function TAllPultKeys.InterceptKey(vCode: Integer): Boolean;
var i: Integer;
begin
  result := false;
  if FEditDlgShowing then
  begin
    ReadFromGrid(FGrid);
    AddPultKey(vCode, '');
    SaveSettings;
    SaveToGrid(FGrid);
    for i := 0 to FGrid.Count - 1 do
      if FGrid.Items[i].Strings[1] = '' then
      begin
        FGrid.Items[i].Focused := True;
        SendMessage(FGrid.Handle, WM_KEYDOWN, VK_RIGHT, 0);
        SendMessage(FGrid.Handle, WM_KEYDOWN, VK_F2, 0);
        Break;
      end;
    result := true;
  end;
end;

procedure TAllPultKeys.ReadFromGrid(aGrid: TdxTreeList);
var i: Integer;
begin
  inherited;
  Clear;
  for i := 0 to aGrid.Count - 1 do
    AddPultKey(
      aGrid.Items[i].Values[0],
      aGrid.Items[i].Strings[1]);
  SaveSettings;
end;

procedure TAllPultKeys.ReadSettings;
var i, cnt: Integer;
begin
  Clear;
  cnt := FIni.ReadInteger('KeysCount', 0);
  for i := 0 to cnt - 1 do
  begin
    AddPultKey(
      FIni.ReadInteger('Code' + IntToStr(i), 0),
      FIni.ReadString('Descr' + IntToStr(i), ''));
  end;
end;

procedure TAllPultKeys.SaveSettings;
var i: Integer;
begin
  inherited;
  FIni.WriteInteger('KeysCount', FPultKeysList.Count);
  for i := 0 to FPultKeysList.Count - 1 do
  begin
    FIni.WriteInteger('Code' + IntToStr(i),
      TPultKey(FPultKeysList.Items[i]).Code);
    FIni.WriteString('Descr' + IntToStr(i),
      TPultKey(FPultKeysList.Items[i]).Descr);
  end;
end;

procedure TAllPultKeys.SaveToGrid(aGrid: TdxTreeList);
var i: Integer;
  aN: TdxTreeListNode;
begin
  inherited;
  for i := 0 to FPultKeysList.Count - 1 do
  begin
    aN := aGrid.Add;
    aN.Values[0] := TPultKey(FPultKeysList.Items[i]).Code;
    aN.Values[1] := TPultKey(FPultKeysList.Items[i]).Descr;
  end;
end;

function TAllPultKeys.ShowEditDlg: Boolean;
var PKDlg: T_PultKeyDlg;
begin
  PKDlg := T_PultKeyDlg.Create(Self);
  FGrid := PKDlg.gKeys;
  SaveToGrid(FGrid);
  FEditDlgShowing := true;
  if PKDlg.ShowModal = mrOK then
  begin
    ReadFromGrid(FGrid);
    SaveSettings;
    Result := true;
  end else Result := false;
  FEditDlgShowing := false;
  PKDlg.Free;
  FGrid := nil;
end;

{ TIniSaveClass }

constructor TIniSaveClass.Create(AOwner: TComponent; aIniFileName,
  aIniSection: string);
begin
  inherited Create(AOwner);
  FIni := TFormStorage.Create(Self);
  FIni.Active := False;
  FIni.IniFileName := aIniFileName;
  FIni.IniSection := aIniSection;
  ReadSettings;
end;

destructor TIniSaveClass.Destroy;
begin
  FIni.Free;
  inherited;
end;

procedure TIniSaveClass.ReadFromGrid(aGrid: TdxTreeList);
begin
  if aGrid = nil then Exit;
end;

procedure TIniSaveClass.ReadSettings;
begin
end;

procedure TIniSaveClass.SaveSettings;
begin
  FIni.EraseSections;
end;

procedure TIniSaveClass.SaveToGrid(aGrid: TdxTreeList);
begin
  if aGrid = nil then Exit;
  aGrid.ClearNodes;
  ReadSettings;
end;

function TIniSaveClass.ShowEditDlg: Boolean;
begin
  Result := False;
end;

{ TGlobalSettings }

constructor TGlobalSettings.Create(AOwner: TComponent;
  aIniFileName: string);
begin
  inherited Create(AOwner, aIniFileName, 'Options');
end;

procedure TGlobalSettings.ReadSettings;
begin
  VolumeInc := FIni.ReadInteger('VolumeInc', 2000);
  Interval := FIni.ReadInteger('Interval', 250);
  SplashTime := FIni.ReadInteger('SplashTime', 1000);
  ShowSplashMsg := FIni.ReadInteger('ShowSplashMsg', 1);
end;

procedure TGlobalSettings.SaveSettings;
begin
  FIni.WriteInteger('VolumeInc', VolumeInc);
  FIni.WriteInteger('Interval', Interval);
  FIni.WriteInteger('SplashTime', SplashTime);
  FIni.WriteInteger('ShowSplashMsg', ShowSplashMsg);
end;

function TGlobalSettings.ShowEditDlg: Boolean;
var OpDlg: T_OptionsDlg;
begin
  OpDlg := T_OptionsDlg.Create(Self);
  OpDlg.spProcVolume.Value := VolumeInc;
  OpDlg.spInterval.Value := Interval;
  OpDlg.spSplashTime.Value := SplashTime;
  OpDlg.cbShowSplashMsg.Checked := (ShowSplashMsg = 1);
  if OpDlg.ShowModal = mrOK then
  begin
    VolumeInc := OpDlg.spProcVolume.Value;
    Interval := OpDlg.spInterval.Value;
    SplashTime := OpDlg.spSplashTime.Value;
    ShowSplashMsg := Ord(OpDlg.cbShowSplashMsg.Checked);
    SaveSettings;
    Result := true;
  end else Result := false;
  OpDlg.Free;
end;

{ TRemoteApp }

constructor TRemoteApp.Create(AOwner: TComponent; aIniFileName: string; aAppName: string);
begin
  AppName := aAppName;
  FSettingList := TObjectList.Create(true);
  inherited Create(AOwner, aIniFileName, 'App ' + AppName);
end;

destructor TRemoteApp.Destroy;
begin
  FSettingList.Clear;
  FSettingList.Free;
  inherited;
end;

function TRemoteApp.GetAppByName(vAppName: string): TRemoteApp;
var i: Integer;
begin
  Result := nil;
  for i := 0 to TRemoteAppList(Owner).FAppList.Count - 1 do
    if vAppName = TRemoteApp(TRemoteAppList(Owner).FAppList.Items[i]).AppName then
    begin
      Result := TRemoteApp(TRemoteAppList(Owner).FAppList.Items[i]);
      Exit;
    end;
end;

function TRemoteApp.GetPultKeyByCode(vCode: Integer): TPultKey;
var i: Integer;
begin
  Result := nil;
  for i := 0 to TRemoteAppList(Owner).FPultKeys.FPultKeysList.Count - 1 do
    if vCode = TPultKey(TRemoteAppList(Owner).FPultKeys.FPultKeysList.Items[i]).Code then
    begin
      Result := TPultKey(TRemoteAppList(Owner).FPultKeys.FPultKeysList.Items[i]);
      Exit;
    end;
end;

function TRemoteApp.ProcessMsg(vCode: Integer): string;
var i, y: Integer;
  Hnd: HWND;
begin
  Result := '';
  if (FKeysPoint[vCode] <> nil) and
    (TRemoteSetting(FKeysPoint[vCode]).PMsgApp <> nil) then
  begin
    Hnd := AppFindWindow(TRemoteSetting(FKeysPoint[vCode]).PMsgApp);
    if Hnd <> 0 then
      for y := 1 to TRemoteSetting(FKeysPoint[vCode]).Rept do
        PostMessage(
          Hnd,
          TRemoteSetting(FKeysPoint[vCode]).Msg,
          TRemoteSetting(FKeysPoint[vCode]).WParam,
          TRemoteSetting(FKeysPoint[vCode]).LParam);
    Result := TRemoteSetting(FKeysPoint[vCode]).Comment;
  end;
end;

procedure TRemoteApp.ReadFromGrid(aGrid: TdxTreeList);
var i: Integer;
  RemS: TRemoteSetting;
begin
  FSettingList.Clear;
  for i := 0 to MaxKeyIndex do FKeysPoint[i] := nil;

  for i := 0 to aGrid.Count - 1 do
  begin
    RemS := TRemoteSetting.Create;
    if aGrid.Items[i].Values[3] <> ''
      then RemS.Msg := aGrid.Items[i].Values[3] else RemS.Msg := 0;
    RemS.PultKey := aGrid.Items[i].Values[0];
    if aGrid.Items[i].Values[5] <> ''
      then RemS.LParam := aGrid.Items[i].Values[5] else RemS.LParam := 0;
    if aGrid.Items[i].Values[4] <> ''
      then RemS.WParam := aGrid.Items[i].Values[4] else RemS.WParam := 0;
    RemS.Comment := aGrid.Items[i].Values[7];
    if aGrid.Items[i].Values[6] <> ''
      then RemS.Rept := aGrid.Items[i].Values[6] else RemS.Rept := 1;
    RemS.MsgApp := aGrid.Items[i].Values[2];
    RemS.PPultKey := GetPultKeyByCode(RemS.PultKey);
    RemS.PMsgApp := GetAppByName(RemS.MsgApp);
    FSettingList.Add(Rems);
    FKeysPoint[RemS.PultKey] := RemS;
  end;
  SaveSettings;
end;

procedure TRemoteApp.ReadSettings;
var i, cnt: Integer;
  RemS: TRemoteSetting;
begin
  inherited;
  FSettingList.Clear;
  for i := 0 to MaxKeyIndex do FKeysPoint[i] := nil;

  if AppName = SelfAppName
    then FilePathName := Application.ExeName
  else FilePathName := FIni.ReadString('FilePathName', '');
  ParamExec := FIni.ReadString('ParamExec', '');
  ClassName := FIni.ReadString('ClassName', '');
  WindowName := FIni.ReadString('WindowName', '');
  cnt := FIni.ReadInteger('SettingsCount', 0);

  for i := 0 to cnt - 1 do
  begin
    RemS := TRemoteSetting.Create;
    RemS.Msg := FIni.ReadInteger('Msg' + IntToStr(i), 0);
    RemS.PultKey := FIni.ReadInteger('PultKey' + IntToStr(i), 0);
    RemS.LParam := FIni.ReadInteger('LParam' + IntToStr(i), 0);
    RemS.WParam := FIni.ReadInteger('WParam' + IntToStr(i), 0);
    RemS.Comment := FIni.ReadString('Comment' + IntToStr(i), '');
    RemS.Rept := FIni.ReadInteger('Rept' + IntToStr(i), 1);
    RemS.MsgApp := FIni.ReadString('MsgApp' + IntToStr(i), '');
    RemS.PPultKey := GetPultKeyByCode(RemS.PultKey);
    RemS.PMsgApp := GetAppByName(RemS.MsgApp);
    FSettingList.Add(Rems);
    FKeysPoint[RemS.PultKey] := RemS;
  end;

end;

procedure TRemoteApp.SaveSettings;
var i: Integer;
begin
  inherited;
  FIni.WriteString('FilePathName', FilePathName);
  FIni.WriteString('ParamExec', ParamExec);
  FIni.WriteString('ClassName', ClassName);
  FIni.WriteString('WindowName', WindowName);
  FIni.WriteInteger('SettingsCount', FSettingList.Count);

  for i := 0 to FSettingList.Count - 1 do
  begin
    FIni.WriteInteger('Msg' + IntToStr(i),
      TRemoteSetting(FSettingList.Items[i]).Msg);
    FIni.WriteInteger('PultKey' + IntToStr(i),
      TRemoteSetting(FSettingList.Items[i]).PultKey);
    FIni.WriteInteger('LParam' + IntToStr(i),
      TRemoteSetting(FSettingList.Items[i]).LParam);
    FIni.WriteInteger('WParam' + IntToStr(i),
      TRemoteSetting(FSettingList.Items[i]).WParam);
    FIni.WriteString('Comment' + IntToStr(i),
      TRemoteSetting(FSettingList.Items[i]).Comment);
    FIni.WriteInteger('Rept' + IntToStr(i),
      TRemoteSetting(FSettingList.Items[i]).Rept);
    FIni.WriteString('MsgApp' + IntToStr(i),
      TRemoteSetting(FSettingList.Items[i]).MsgApp);
  end;
  //    PPultKey: TPultKey;
  //    PMsgApp: TRemoteApp;

end;

procedure TRemoteApp.SaveToGrid(aGrid: TdxTreeList);
var i: Integer;
  aN: TdxTreeListNode;
  y: Integer;
begin
  inherited;
  for i := 0 to TRemoteAppList(Owner).FPultKeys.FPultKeysList.Count - 1 do
  begin
    aN := aGrid.Add;
    aN.Values[0] := TPultKey(TRemoteAppList(Owner).FPultKeys.FPultKeysList.Items[i]).Code;
    aN.Values[1] := TPultKey(TRemoteAppList(Owner).FPultKeys.FPultKeysList.Items[i]).Descr;
    aN.Values[6] := 1;

    for y := 0 to FSettingList.Count - 1 do
      if aN.Values[0] = TRemoteSetting(FSettingList.Items[y]).PultKey then
      begin
        aN.Values[2] := TRemoteSetting(FSettingList.Items[y]).MsgApp;
        aN.Values[3] := TRemoteSetting(FSettingList.Items[y]).Msg;
        aN.Values[4] := TRemoteSetting(FSettingList.Items[y]).WParam;
        aN.Values[5] := TRemoteSetting(FSettingList.Items[y]).LParam;
        aN.Values[6] := TRemoteSetting(FSettingList.Items[y]).Rept;
        aN.Values[7] := TRemoteSetting(FSettingList.Items[y]).Comment;
        Break;
      end;
  end;
end;

{ TRemoteAppList }

procedure TRemoteAppList.AddAppToList(vApp: TRemoteApp);
begin
  FAppList.Add(vApp);
end;

procedure TRemoteAppList.AddAppToList(vAppName, vFilePathName, vParamExec,
  vClassName, vWindowName: string);
var vApp: TRemoteApp;
begin
  vApp := TRemoteApp.Create(Self, FIni.IniFileName, vAppName);
  vApp.FilePathName := vFilePathName;
  vApp.ParamExec := vParamExec;
  vApp.ClassName := vClassName;
  vApp.WindowName := vWindowName;
  AddAppToList(vApp);
end;

constructor TRemoteAppList.Create(AOwner: TComponent;
  aIniFileName: string; vPultKeys: TAllPultKeys);
begin
  FAppList := TObjectList.Create(true);
  FPultKeys := vPultKeys;
  inherited Create(AOwner, aIniFileName, 'Apps');
  if FAppList.Count > 0 then
  begin
    CurrentApp := TRemoteApp(FAppList.Items[0]);
    CurrentAppIndex := 0;
  end;
end;

destructor TRemoteAppList.Destroy;
begin
  FAppList.Clear;
  FAppList.Free;
  inherited;
end;

function TRemoteAppList.ProcessMsg(vCode: Integer): string;
begin
  if CurrentApp = nil
    then Result := ''
  else Result := CurrentApp.ProcessMsg(vCode);
end;

procedure TRemoteAppList.ReadSettings;
var i, cnt: Integer;
  RemApp: TRemoteApp;
begin
  inherited;
  FLastOpenDir := FIni.ReadString('LastOpenDir', ExtractFilePath(Application.ExeName));
  FAppList.Clear;
  cnt := FIni.ReadInteger('AppCount', 0);
  for i := 0 to cnt - 1 do
  begin
    RemApp := TRemoteApp.Create(Self, FIni.IniFileName,
      FIni.ReadString('App' + IntToStr(i), ''));
    AddAppToList(RemApp);
  end;

  for i := 0 to FAppList.Count - 1 do
  begin
    TRemoteApp(FAppList.Items[i]).ReadSettings;
  end;

  if cnt = 0 then //  
  begin
    AddAppToList(SelfAppName,
      Application.ExeName, '', 'T_RSignalMAPI', '');
  end;
end;

procedure TRemoteAppList.SaveSettings;
var i: Integer;
begin
  inherited;
  SaveLastDir;
  FIni.WriteInteger('AppCount', FAppList.Count);
  for i := 0 to FAppList.Count - 1 do
    FIni.WriteString('App' + IntToStr(i),
      TRemoteApp(FAppList.Items[i]).AppName);
  for i := 0 to FAppList.Count - 1 do
    TRemoteApp(FAppList.Items[i]).SaveSettings;
end;

procedure TRemoteAppList.SaveLastDir;
begin
  FIni.WriteString('LastOpenDir', FLastOpenDir);
end;

function TRemoteAppList.ShowEditDlg: Boolean;
begin
  if FRemAppDlg <> nil then Exit;
  Result := True;
  FRemAppDlg := T_RemAppListDlg.Create(Self);
  LoadAppsToCombo;
  LoadAppInfoToForm;
  FRemAppDlg.cAppSelect.OnChange := ComboChange;
  FRemAppDlg.AcNew.OnExecute := AcNew;
  FRemAppDlg.AcSave.OnExecute := AcSave;
  FRemAppDlg.AcDel.OnExecute := AcDel;
  if FRemAppDlg.ShowModal = mrOK then
  begin
    LoadAppInfoFromForm;
    SaveSettings;
    Result := True;
  end;
  FRemAppDlg.Free;
  FRemAppDlg := nil;
end;

procedure TRemoteAppList.LoadAppsToCombo;
var i: Integer;
begin
  FRemAppDlg.cAppSelect.Clear;
  for i := 0 to FAppList.Count - 1 do
    FRemAppDlg.cAppSelect.Items.Add(TRemoteApp(FAppList.Items[i]).AppName);
  FRemAppDlg.cAppSelect.ItemIndex := 0;
end;

procedure TRemoteAppList.LoadAppInfoToForm;
var i: Integer;
begin
  if FRemAppDlg = nil then Exit;
  i := FRemAppDlg.cAppSelect.ItemIndex;
  FRemAppDlg.cFilePathName.Text :=
    TRemoteApp(FAppList.Items[i]).FilePathName;
  FRemAppDlg.cParamExec.Text :=
    TRemoteApp(FAppList.Items[i]).ParamExec;
  FRemAppDlg.cClassName.Text :=
    TRemoteApp(FAppList.Items[i]).ClassName;
  FRemAppDlg.cWindowName.Text :=
    TRemoteApp(FAppList.Items[i]).WindowName;
  TRemoteApp(FAppList.Items[i]).SaveToGrid(FRemAppDlg.gMsg);
end;

procedure TRemoteAppList.ComboChange(Sender: TObject);
begin
  LoadAppInfoToForm;
end;

procedure TRemoteAppList.AcNew(Sender: TObject);
var vStr: string;
  i: Integer;
begin
  if not InputQuery('', '', vStr) then Exit;
  vStr := Trim(vStr);
  for i := 0 to FAppList.Count - 1 do
    if AnsiUpperCase(TRemoteApp(FAppList.Items[i]).AppName) = AnsiUpperCase(vStr) then
    begin
      MessageDlg('   !', mtError, [mbOK], 0);
      Exit;
    end;
  AddAppToList(vStr, '', '', '', '');
  LoadAppsToCombo;
  FRemAppDlg.cAppSelect.ItemIndex := FRemAppDlg.cAppSelect.Items.Count - 1;
  LoadAppInfoToForm;
end;

procedure TRemoteAppList.AcSave(Sender: TObject);
begin
  LoadAppInfoFromForm;
end;

procedure TRemoteAppList.LoadAppInfoFromForm;
var i: Integer;
begin
  if FRemAppDlg = nil then Exit;
  i := FRemAppDlg.cAppSelect.ItemIndex;
  TRemoteApp(FAppList.Items[i]).FilePathName :=
    FRemAppDlg.cFilePathName.FileName;
  TRemoteApp(FAppList.Items[i]).ParamExec :=
    FRemAppDlg.cParamExec.Text;
  TRemoteApp(FAppList.Items[i]).ClassName :=
    FRemAppDlg.cClassName.Text;
  TRemoteApp(FAppList.Items[i]).WindowName :=
    FRemAppDlg.cWindowName.Text;
  TRemoteApp(FAppList.Items[i]).SaveSettings;
  TRemoteApp(FAppList.Items[i]).ReadFromGrid(FRemAppDlg.gMsg);
end;

procedure TRemoteAppList.SelectRunApp(vSelCtrl: TSelectCtrl);
var vStr: string;
begin
  if (vSelCtrl <> scCancel)
    and (FSelectRunApp = nil) then
  begin
    FSelectRunApp := T_SelectRunApp.Create(Self);
    FSelectRunApp.SelectMode := smFile; //TSelectMode(Ord(vSelCtrl) > 3);
    FSelectRunApp.Show;
    SelectDlgShowing := true;
    FillAppItems;
    if FSelectRunApp.SelectMode = smFile then
    begin
      FSelectRunApp.cFilesList.DirectoryName := FLastOpenDir;
    end;
    FSelectRunApp.cAppList.SetSelected(CurrentAppIndex);
    Exit;
  end;
  case vSelCtrl of
    scNext: FSelectRunApp.cAppList.Next;
    scPrev: FSelectRunApp.cAppList.Prev;
    scCancel: CloseSelectRunAppDlg;
    scSelect:
      begin
        {if FSelectRunApp.SelectMode = smApp then
        begin}
        ChangeToApp(FSelectRunApp.cAppList.GetSelectedIndex + 1);
        CloseSelectRunAppDlg;
        //end;
      end;
    scFileNext: if FSelectRunApp.SelectMode = smFile then FSelectRunApp.cFilesList.Next;
    scFilePrev: if FSelectRunApp.SelectMode = smFile then FSelectRunApp.cFilesList.Prev;
    scFileSelect:
      begin
        if FSelectRunApp.SelectMode = smFile then
        begin
          if FSelectRunApp.cFilesList.OpenDirItem then
          begin
            vStr := FSelectRunApp.cFilesList.GetSelectedFileName;
            if vStr <> '' then
            begin
              FLastOpenDir := FSelectRunApp.cFilesList.DirectoryName;
              SaveLastDir;
              ShellExecute(0, 'open',
                PChar(ExtractFileName(TRemoteApp(FAppList[FSelectRunApp.cAppList.GetSelectedIndex + 1]).FilePathName)),
                '',
                PChar(ExtractFilePath(TRemoteApp(FAppList[FSelectRunApp.cAppList.GetSelectedIndex + 1]).FilePathName)),
                SW_SHOWNORMAL);
              Sleep(2000);
              ShellExecute(0, 'open',
                PChar(ExtractFileName(TRemoteApp(FAppList[FSelectRunApp.cAppList.GetSelectedIndex + 1]).FilePathName)),
                PChar('"' + vStr + '"'),
                PChar(ExtractFilePath(TRemoteApp(FAppList[FSelectRunApp.cAppList.GetSelectedIndex + 1]).FilePathName)),
                SW_SHOWNORMAL);
              ChangeToApp(FSelectRunApp.cAppList.GetSelectedIndex + 1);
              CloseSelectRunAppDlg;
            end;
          end;
        end;
      end;
    scFileExec:
      begin
        if FSelectRunApp.SelectMode = smFile then
        begin
          vStr := FSelectRunApp.cFilesList.GetSelectedFileName;
          if vStr <> '' then
          begin
            FLastOpenDir := FSelectRunApp.cFilesList.DirectoryName;
            ShellExecute(0, 'open', PChar(ExtractFileName(vStr)),
              '', PChar(ExtractFilePath(vStr)), SW_SHOWNORMAL);
            CloseSelectRunAppDlg;
          end;
        end;
      end;
  end;
end;

procedure TRemoteAppList.CloseSelectRunAppDlg;
begin
  FSelectRunApp.Free;
  FSelectRunApp := nil;
  SelectDlgShowing := false;
end;

procedure TRemoteAppList.FillAppItems;
var i: Integer;
begin
  FSelectRunApp.cAppList.ClearItems;
  for i := 1 to FAppList.Count - 1 do
  begin
    FSelectRunApp.cAppList.AddFile(
      TRemoteApp(FAppList.Items[i]).FilePathName,
      TRemoteApp(FAppList.Items[i]).AppName);
  end;
end;

procedure TRemoteAppList.ChangeToApp(vIndex: Integer);
var ic: TIcon;
  IconIndex: Word;
  pstr: array[0..255] of char;
begin
  if CurrentAppIndex <> 0 then
  begin
    CloseCurrentApp;
  end;
  CurrentAppIndex := vIndex;
  CurrentApp := TRemoteApp(FAppList.Items[vIndex]);
  if vIndex <> 0 then
  begin
    AppTestRun(CurrentApp);
    FTrayIcon := TRxTrayIcon.Create(Self);
    ic := TIcon.Create;
    StrPCopy(pstr, TRemoteApp(FAppList.Items[vIndex]).FilePathName);
    ic.Handle := ExtractAssociatedIcon(hInstance, pstr, IconIndex);
    FTrayIcon.Icon.Assign(ic);
    FTrayIcon.Active := True;
    FTrayIcon.Hint := 'Remote Control: ' + TRemoteApp(FAppList.Items[vIndex]).AppName;
    ic.Free;
  end;
end;

procedure TRemoteAppList.CloseCurrentApp;
begin
  SendMessage(AppFindWindow(CurrentApp), WM_CLOSE, 0, 0);
  Sleep(2000);
  if FTrayIcon <> nil then
  begin
    FTrayIcon.Active := False;
    FTrayIcon.Free;
    FTrayIcon := nil;
  end;
  sProcesses.CloseProcess(TRemoteApp(FAppList.Items[CurrentAppIndex]).FilePathName);
  CurrentAppIndex := 0;
  CurrentApp := TRemoteApp(FAppList.Items[0]);
end;

procedure TRemoteAppList.HelpDlg;
var i: integer;
  vKey, vApp, vComment: string;
  function AddRet(vStr: string; vRetCnt: Integer): string;
  var i: Integer;
  begin
    Result := vStr;
    if Length(vStr) < vRetCnt then
      for i := 1 to vRetCnt - Length(vStr)
        do Result := Result + ' ';
  end;
begin
  if FHelp = nil then
  begin
    FHelp := T_Help.Create(Self);
    FHelp.KeysBox.Clear;
    for i := 0 to TRemoteApp(FAppList.Items[CurrentAppIndex]).FSettingList.Count - 1 do
    begin
      vKey := TRemoteSetting(TRemoteApp(FAppList.Items[CurrentAppIndex]).FSettingList.Items[i]).PPultKey.Descr;
      vApp := TRemoteSetting(TRemoteApp(FAppList.Items[CurrentAppIndex]).FSettingList.Items[i]).MsgApp;
      vComment := TRemoteSetting(TRemoteApp(FAppList.Items[CurrentAppIndex]).FSettingList.Items[i]).Comment;
      FHelp.KeysBox.Lines.Add('  ' + AddRet(vKey, 20) + #9 + vApp + #9 + vComment);
    end;
    FHelp.Show;
  end else begin
    FHelp.Free;
    FHelp := nil;
  end;
end;

procedure TRemoteAppList.AcDel(Sender: TObject);
begin
  if FRemAppDlg.cAppSelect.ItemIndex = 0 then
  begin
    MessageDlg('   !', mtError, [mbOk], 0);
    Exit;
  end;
  if MessageDlg('  "' +
    FRemAppDlg.cAppSelect.Text + '"   ?', mtConfirmation,
    [mbYes, mbNo], 0) <> mrYes then Exit;
  FAppList.Delete(FRemAppDlg.cAppSelect.ItemIndex);
  SaveSettings;
  LoadAppsToCombo;
  LoadAppInfoToForm;
end;

end.

